@@ -75,18 +75,17 @@ namespace Beagle.Filters {
 
 		public FilterMPlayerVideo ()
 		{
+			// 1: Priority update after FilterVideo was added
+			SetVersion (1);
+
 			PreLoad = false;
 			SetFileType ("video");
 		}
 
 		protected override void RegisterSupportedTypes ()
 		{
-			foreach (string s in mime_types) {
-				FilterFlavor flavor = FilterFlavor.NewFromMimeType (s);
-				flavor.Priority = -1; // Prefer Totem over this one
-
-				AddSupportedFlavor (flavor);
-			}
+			foreach (string s in mime_types)
+				AddSupportedFlavor (FilterFlavor.NewFromMimeType (s));
 		}
 		
 		private string AspectString(float aspect) {
