@@ -27,6 +27,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 
 using GMime;
@@ -37,14 +38,21 @@ using Beagle.Util;
 
 namespace Beagle.Filters {
 
-	[PropertyKeywordMapping (Keyword="mailfrom",     PropertyName="fixme:from_name",    IsKeyword=false)]
-	[PropertyKeywordMapping (Keyword="mailfromaddr", PropertyName="fixme:from_address", IsKeyword=false)]
-	[PropertyKeywordMapping (Keyword="mailto",       PropertyName="fixme:to_name",      IsKeyword=false)]
-	[PropertyKeywordMapping (Keyword="mailtoaddr",   PropertyName="fixme:to_address",   IsKeyword=false)]
-	[PropertyKeywordMapping (Keyword="mailinglist",  PropertyName="fixme:mlist",        IsKeyword=true, Description="Mailing list id")]
+	[PropertyKeywordMapping (Keyword="mailfrom",     PropertyName="fixme:from_name",    IsKeyword=false, Description="Name of email sender")]
+	[PropertyKeywordMapping (Keyword="mailfrom",     PropertyName="parent:fixme:from_name",    IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailfromaddr", PropertyName="fixme:from_address", IsKeyword=false, Description="Email address of sender")]
+	[PropertyKeywordMapping (Keyword="mailfromaddr", PropertyName="parent:fixme:from_address", IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailto",       PropertyName="fixme:to_name",      IsKeyword=false, Description="Name of receipient")]
+	[PropertyKeywordMapping (Keyword="mailto",       PropertyName="parent:fixme:to_name",      IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailtoaddr",   PropertyName="fixme:to_address",   IsKeyword=false, Description="Email address of receipient")]
+	[PropertyKeywordMapping (Keyword="mailtoaddr",   PropertyName="parent:fixme:to_address",   IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="mailinglist",  PropertyName="fixme:mlist",        IsKeyword=false, Description="Mailing list id e.g. dashboard-hackers.gnome.org")]
+	[PropertyKeywordMapping (Keyword="mailinglist",  PropertyName="parent:fixme:mlist",        IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="inattachment",  PropertyName="parent:fixme:hasAttachments", IsKeyword=true, Description="Use 'inattachment:true' for email attachments.")]
 	public class FilterMail : Beagle.Daemon.Filter, IDisposable {
 
 		private static bool gmime_initialized = false;
+		private static bool gmime_broken = false;
 
 		private GMime.Message message;
 		private PartHandler handler;
@@ -56,26 +64,57 @@ namespace Beagle.Filters {
 			// 2: No need to separately add sanitized version of emails.
 			//    BeagleAnalyzer uses a tokenfilter taking care of this.
 			// 3: Add standard file properties to attachments.
-			SetVersion (3);
+			// 4: Store snippets
+			// 5: Store mailing list id as tokenized
+			SetVersion (5);
+
+			SnippetMode = true;
+			SetFileType ("mail");
 		}
 
 		protected override void RegisterSupportedTypes ()
 		{
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("message/rfc822"));
+
+			// Add the list of user requested maildir directories
+			// This is useful if beagle (xdgmime) does not correctly detect the mimetypes
+			// of several maildir files as message/rfc822
+
+			List<string[]> values = Conf.Daemon.GetListOptionValues (Conf.Names.Maildirs);
+			if (values == null)
+				return;
+
+			foreach (string[] maildir in values) {
+				FilterFlavor flavor =
+					new FilterFlavor (new Uri (maildir [0] + "/*").ToString (),
+							  null, // No meaning of extension for maildir files
+							  null,
+							  1 /* Should be more than priority of text filter */);
+				AddSupportedFlavor (flavor);
+			}
 		}
 
 		protected override void DoOpen (FileInfo info)
 		{
 			if (!gmime_initialized) {
+				gmime_initialized = true;
+
+				// Only try initialization once; if it fails, it'll never work.
 				try {
 					GMime.Global.Init ();
-					gmime_initialized = true;
-				} catch {
+				} catch (Exception e) {
+					Log.Error (e, "Unable to initialize GMime");
+					gmime_broken = true;
 					Error ();
 					return;
 				}
 			}
 
+			if (gmime_broken) {
+				Error ();
+				return;
+			}
+
 			int mail_fd = Mono.Unix.Native.Syscall.open (info.FullName, Mono.Unix.Native.OpenFlags.O_RDONLY);
 			
 			if (mail_fd == -1)
@@ -156,17 +195,15 @@ namespace Beagle.Filters {
 				AddProperty (Property.NewUnsearched ("fixme:reference", refs.Msgid));
 
 			string list_id = this.message.GetHeader ("List-Id");
-			if (list_id != null) {
-				// FIXME: Might need some additional parsing.
-				AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
-			}
+			if (list_id != null)
+				AddProperty (Property.New ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
 
 			// KMail can store replies in the same folder
 			// Use issent flag to distinguish between incoming
 			// and outgoing message
 			string kmail_msg_sent = this.message.GetHeader ("X-KMail-Link-Type");
 			bool issent_is_set = false;
-			foreach (Property property in IndexableProperties) {
+			foreach (Property property in Indexable.Properties) {
 				if (property.Key == "fixme:isSent") {
 					issent_is_set = true;
 					break;
@@ -178,11 +215,11 @@ namespace Beagle.Filters {
 
 		protected override void DoPullSetup ()
 		{
-			this.handler = new PartHandler (this);
+			this.handler = new PartHandler (Indexable);
 			using (GMime.Object mime_part = this.message.MimePart)
 				this.handler.OnEachPart (mime_part);
 
-			AddChildIndexables (this.handler.ChildIndexables);
+			AddIndexables (this.handler.ChildIndexables);
 		}
 
 		protected override void DoPull ()
@@ -192,6 +229,11 @@ namespace Beagle.Filters {
 				return;
 			}
 
+			if (handler.HtmlPart) {
+				DoPullingReaderPull ();
+				return;
+			}
+
 			string l = handler.Reader.ReadLine ();
 
 			if (l == null)
@@ -202,6 +244,23 @@ namespace Beagle.Filters {
 			}
 		}
 
+		char[] pulling_reader_buf = null;
+		const int BUFSIZE = 1024;
+		private void DoPullingReaderPull ()
+		{
+			if (pulling_reader_buf == null)
+				pulling_reader_buf = new char [BUFSIZE];
+
+			int count = handler.Reader.Read (pulling_reader_buf, 0, BUFSIZE);
+			if (count == 0) {
+				Finished ();
+				pulling_reader_buf = null;
+				return;
+			}
+
+			AppendChars (pulling_reader_buf, 0, count);
+		}
+
 		protected override void DoClose ()
 		{
 			Dispose ();
@@ -211,6 +270,7 @@ namespace Beagle.Filters {
 		{
 			if (this.handler != null && this.handler.Reader != null)
 				this.handler.Reader.Close ();
+
 			this.handler = null;
 
 			if (this.message != null) {
@@ -220,12 +280,17 @@ namespace Beagle.Filters {
 		}
 
 		private class PartHandler {
-			private Beagle.Daemon.Filter filter;
+			private Indexable indexable;
 			private int count = 0; // parts handled so far
 			private int depth = 0; // part recursion depth
 			private ArrayList child_indexables = new ArrayList ();
 			private TextReader reader;
 
+			private bool html_part = false;
+			internal bool HtmlPart {
+				get { return html_part; }
+			}
+
 			// Blacklist a handful of common MIME types that are
 			// either pointless on their own or ones that we don't
 			// have filters for.
@@ -237,9 +302,9 @@ namespace Beagle.Filters {
 				"text/x-vcard"
 			};
 
-			public PartHandler (Beagle.Daemon.Filter filter)
+			public PartHandler (Indexable parent_indexable)
 			{
-				this.filter = filter;
+				this.indexable = parent_indexable;
 			}
 
 			private bool IsMimeTypeHandled (string mime_type)
@@ -324,6 +389,41 @@ namespace Beagle.Filters {
 							no_child_needed = true;
 
 							this.reader = new StreamReader (stream);
+						} else if (mime_type == "text/html") {
+							no_child_needed = true;
+							html_part = true;
+							string enc = part.GetContentTypeParameter ("charset"); 
+							// DataWrapper.Stream is a very limited stream
+							// and does not allow Seek or Tell
+							// HtmlFilter requires Stream.Position=0.
+							// Play safe and create a memorystream
+							// for HTML parsing.
+
+							GMime.StreamMem mem_stream;
+							mem_stream = new GMime.StreamMem ();
+
+							GMime.Stream data_stream;
+							data_stream = ((StreamWrapper) stream).GMimeStream;
+							data_stream.WriteToStream (mem_stream);
+							data_stream.Flush ();
+
+							// The StreamWrapper and hence the memory_stream
+							// will be closed when the reader is closed
+							// after Pull()-ing is done.
+							System.IO.Stream html_stream; 
+							html_stream = new StreamWrapper (mem_stream);
+							html_stream.Seek (0, SeekOrigin.Begin);
+
+							stream.Close ();
+
+							try {
+								this.reader = FilterHtml.GetHtmlReader (html_stream, enc);
+							} catch (Exception e) {
+								Log.Debug (e, "Exception while filtering HTML email {0}", this.indexable.Uri);
+								this.reader = null;
+								html_stream.Close ();
+								html_part = false;
+							}
 						}
 					}
 
@@ -334,14 +434,21 @@ namespace Beagle.Filters {
 						// attachments along with (real) attachments.
 
 						if (Array.IndexOf (blacklisted_mime_types, mime_type) == -1) {
-							string sub_uri = this.filter.Uri.ToString () + "#" + this.count;
+							string sub_uri = this.indexable.Uri.ToString () + "#" + this.count;
 							Indexable child = new Indexable (new Uri (sub_uri));
 
-							child.DisplayUri = new Uri (this.filter.DisplayUri.ToString () + "#" + this.count);
+							child.DisplayUri = new Uri (this.indexable.DisplayUri.ToString () + "#" + this.count);
 
+							// This is a special case.
+							// Even for mails found on disk, MailMessage hitype is set
 							child.HitType = "MailMessage";
 							child.MimeType = mime_type;
-							child.CacheContent = false;
+
+							// If this is the richest part we found for multipart emails, add its content to textcache
+							if (this.depth == 1 && this.count == 0)
+								child.CacheContent = true;
+							else
+								child.CacheContent = false;
 
 							string filename = ((GMime.Part) part).Filename;
 
@@ -357,10 +464,13 @@ namespace Beagle.Filters {
 							else
 								child.SetBinaryStream (stream);
 
+							child.SetChildOf (this.indexable);
+							child.StoreStream ();
+							child.CloseStreams ();
 							this.child_indexables.Add (child);
 						} else {
 							Log.Debug ("Skipping attachment {0}#{1} with blacklisted mime type {2}",
-								   this.filter.Uri, this.count, mime_type);
+								   this.indexable.Uri, this.count, mime_type);
 						}
 					}
 
@@ -382,7 +492,6 @@ namespace Beagle.Filters {
 			}
 		}
 
-				       
 	}
 
 }
