@@ -41,6 +41,12 @@ namespace Beagle.Filters {
 		StreamReader reader;
 		
 		public FilterMan ()
+		{
+			SnippetMode = true;
+			SetFileType ("documentation");
+		}
+
+		protected override void RegisterSupportedTypes ()
 		{
 			// Make this a general troff filter.
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/x-troff-man"));
@@ -48,8 +54,6 @@ namespace Beagle.Filters {
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/x-troff"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/x-troff"));
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/troff"));
-
-			SnippetMode = true;
 		}
  		/*
  			FIXME: 
@@ -96,9 +100,17 @@ namespace Beagle.Filters {
 					}
                       		} else {
                       			// A "regular" string
-
-                      			// FIXME: We need to strip out other macros
-					// (.SH for example)
+                      			// Strip out some of the more common
+                      			// troff macros, ideally we should handle more
+								str = str.Replace (".SH","");
+								str = str.Replace (".PP","");
+								str = str.Replace ("\fI","");
+								str = str.Replace ("\fR","");
+								str = str.Replace (".SH","");
+								str = str.Replace (".B","");
+								str = str.Replace (".TP","");
+								str = str.Replace (".BR","");
+								str = str.Replace ("\fB","");
                       			AppendText (str);
 				
                       		}
