@@ -114,12 +114,13 @@ namespace Beagle.Util {
 		[DllImport ("libbeagleglue.so")]
 		extern static unsafe int screensaver_glue_init ();
 
+		/*
+		 * BeagleDaemon needs to monitor screensaver status
+		 * for faster scheduling when user is idle.
+		 * IndexHelper does not need to monitor screensaver status.
+		 * XssInit is only called from the BeagleDaemon.
+		 */
 		public static bool XssInit ()
-		{
-			return XssInit (false);
-		}
-
-		public static bool XssInit (bool actually_init_xss)
 		{
 			int has_xss = screensaver_glue_init ();
 			use_screensaver = (has_xss == 1);
@@ -137,6 +138,12 @@ namespace Beagle.Util {
 			if (! use_screensaver)
 				return;
 
+			if (! Conf.Daemon.GetOption (Conf.Names.IndexFasterOnScreensaver, true)) {
+				cached_screensaver_running = false;
+				cached_screensaver_idle_time = 0;
+				return;
+			}
+
 			if ((DateTime.Now - screensaver_time).TotalSeconds < screensaver_poll_delay)
 				return;
 
@@ -178,13 +185,14 @@ namespace Beagle.Util {
 
 		///////////////////////////////////////////////////////////////
 
-		const double acpi_poll_delay = 30;
+		public const double acpi_poll_delay = 30;
 		const string ac_present_string = "on-line";
 
 		private static readonly string[] proc_ac_state_filenames = new string[] {
 			"/proc/acpi/ac_adapter/AC/state",
 			"/proc/acpi/ac_adapter/AC0/state",
-			"/proc/acpi/ac_adapter/ADp1/state"
+			"/proc/acpi/ac_adapter/ADp1/state",
+			"/proc/acpi/ac_adapter/ACAD/state"
 		};
 
 		private static string proc_ac_state_filename = null;
