@@ -27,10 +27,13 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Diagnostics;
 using System.IO;
 using System.Net;
+using System.Text.RegularExpressions;
 using System.Threading;
+using System.Reflection;
 
 using System.Xml;
 using System.Xml.Serialization;
@@ -45,6 +48,10 @@ using Beagle.Util;
 using Stopwatch = Beagle.Util.Stopwatch;
 using FSQ = Beagle.Daemon.FileSystemQueryable.FileSystemQueryable;
 
+// Assembly information
+[assembly: AssemblyTitle ("beagle-build-index")]
+[assembly: AssemblyDescription ("Build an index.")]
+
 namespace Beagle.Daemon 
 {
 	class BuildIndex 
@@ -63,10 +70,13 @@ namespace Beagle.Daemon
 		static string [] allowed_files = {
 			"FileAttributesStore.db",
 			"fingerprint",
-			"version"
+			"version",
+			"filterver.dat",
+			"StaticIndex.xml"
 		};
 		
 		static string [] allowed_dirs = {
+			"config",
 			"Locks",
 			"PrimaryIndex",
 			"SecondaryIndex",
@@ -82,11 +92,12 @@ namespace Beagle.Daemon
 
 		static bool indexing = true, restart = false;
 
-		static ArrayList allowed_patterns = new ArrayList ();
-		static ArrayList denied_patterns = new ArrayList ();
+		static Regex allowed_regex = null;
+		static Regex denied_regex = null;
 
 		static Queue pending_files = new Queue ();
 		static Queue pending_directories = new Queue ();
+		static IndexerRequest pending_request;
 		
 		const int BATCH_SIZE = 30;
 		
@@ -108,7 +119,10 @@ namespace Beagle.Daemon
 
 			if (args.Length < 2)
 				PrintUsage ();
-		
+
+			ArrayList allowed_patterns = new ArrayList ();
+			ArrayList denied_patterns = new ArrayList ();
+
 			int i = 0;
 			while (i < args.Length) {
 			
@@ -165,10 +179,10 @@ namespace Beagle.Daemon
 
 					if (next_arg.IndexOf (',') != -1) {
 						foreach (string pattern in next_arg.Split (','))
-							allowed_patterns.Add (new ExcludeItem (ExcludeType.Pattern, pattern));
+							allowed_patterns.Add (pattern);
 						
 					} else {
-						allowed_patterns.Add (new ExcludeItem (ExcludeType.Pattern, next_arg));
+						allowed_patterns.Add (next_arg);
 					}
 					
 					++i;
@@ -180,10 +194,10 @@ namespace Beagle.Daemon
 
 					if (next_arg.IndexOf (',') != -1) {
 						foreach (string pattern in next_arg.Split (','))
-							denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, pattern));
+							denied_patterns.Add (pattern);
 
 					} else {
-						denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, next_arg));
+						denied_patterns.Add (next_arg);
 					}
 
 					++i;
@@ -221,6 +235,10 @@ namespace Beagle.Daemon
 				Environment.Exit (1);
 			}
 
+			// Set the storage dir, this should be used to store log messages
+			// and filterver.dat
+			PathFinder.StorageDir = arg_output;
+
 			foreach (FileSystemInfo info in pending_directories) {
 				if (Path.GetFullPath (arg_output) == info.FullName) {
 					Logger.Log.Error ("Target directory cannot be one of the source paths.");
@@ -260,18 +278,63 @@ namespace Beagle.Daemon
 				}
 			}
 
+			string config_file_path = Path.Combine (arg_output, "StaticIndex.xml");
+			string prev_source = null;
+			if (File.Exists (config_file_path)) {
+				Config static_index_config = Conf.LoadFrom (config_file_path);
+				if (static_index_config == null) {
+					Log.Error ("Invalid configuation file {0}", config_file_path);
+					Environment.Exit (1);
+				}
+
+				prev_source = static_index_config.GetOption ("Source", null);
+				if (arg_source != null && prev_source != arg_source) {
+					Log.Error ("Source already set to {0} for existing static index. Cannot set source to {1}.", prev_source, arg_source);
+					Environment.Exit (1);
+				}
+
+				// If arg_source is not given, and prev_source is present, use prev_source
+				// as the arg_source. This is useful for re-running build-index without
+				// giving --arg_source for already existing static index
+				arg_source = prev_source;
+			}
+
+			if (arg_source == null) {
+				DirectoryInfo dir = new DirectoryInfo (StringFu.SanitizePath (arg_output));
+				arg_source = dir.Name;
+			}
+
 			if (SystemInformation.UsingBattery && arg_disable_on_battery) {
 				Log.Always ("Indexer is disabled when on battery power (--disable-on-battery)");
 				Environment.Exit (0);
 			}
 
-			// Setup some exclude patterns akin to FSQ/FileNameFilter.cs
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*~"));
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "#*#"));
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.o"));
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.a"));
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.aux"));
-			denied_patterns.Add (new ExcludeItem (ExcludeType.Pattern, "*.tmp"));
+			string global_files_config = Path.Combine (Conf.GlobalDir, Conf.Names.FilesQueryableConfig + ".xml");
+			if (! File.Exists (global_files_config)) {
+				Log.Error ("Global configuration file not found {0}", global_files_config);
+				Environment.Exit (0);
+			}
+
+			// Setup regexes for allowed/denied patterns
+			if (allowed_patterns.Count > 0) {
+				allowed_regex = StringFu.GetPatternRegex (allowed_patterns);
+			} else {
+				// Read the exclude values from config
+				// For system-wide indexes, only the global config value will be used
+				Config config = Conf.Get (Conf.Names.FilesQueryableConfig);
+				List<string[]> values = config.GetListOptionValues (Conf.Names.ExcludePattern);
+				if (values != null)
+					foreach (string[] exclude in values)
+						denied_patterns.Add (exclude [0]);
+
+				if (denied_patterns.Count > 0)
+					denied_regex = StringFu.GetPatternRegex (denied_patterns);
+			}
+
+			if (arg_disable_directories && arg_delete) {
+				Log.Warn ("--enable-deletion is ignored as --disable-directories is used");
+				arg_delete = false;
+			}
 
 			Log.Always ("Starting beagle-build-index (pid {0}) at {1}", Process.GetCurrentProcess ().Id, DateTime.Now);
 
@@ -311,6 +374,19 @@ namespace Beagle.Daemon
 			watch.Stop ();
 			Logger.Log.Debug ("Elapsed time {0}.", watch);
 
+			// Write this after indexing is done. This is because, if creating a new index,
+			// LuceneIndexingDriver.Create() is called which purges the entire directory.
+
+			if (prev_source == null) {
+				Config static_index_config = Conf.LoadNew ("StaticIndex.xml");
+
+				// Write StaticIndex.xml containing:
+				// The name of the source
+				static_index_config.SetOption ("Source", arg_source);
+				static_index_config ["Source"].Description = "Source of the static index";
+				Conf.SaveTo (static_index_config, config_file_path);
+			}
+
 			if (restart) {
 				Logger.Log.Debug ("Restarting beagle-build-index");
 				Process p = new Process ();
@@ -329,12 +405,12 @@ namespace Beagle.Daemon
 		
 		static void IndexWorker ()
 		{
-			Log.Debug ("Starting IndexWorker");
+			Logger.Log.Debug ("Starting IndexWorker");
 
 			try {
 				DoIndexing ();
 			} catch (Exception e) {
-				Log.Debug ("Encountered exception while indexing: {0}", e);
+				Logger.Log.Debug ("Encountered exception while indexing: {0}", e);
 			}
 
 			Logger.Log.Debug ("IndexWorker Done");
@@ -347,7 +423,6 @@ namespace Beagle.Daemon
 			int count_files = 0;
 
 			Indexable indexable;
-			IndexerRequest pending_request;
 			pending_request = new IndexerRequest ();
 			Queue modified_directories = new Queue ();
 			
@@ -355,7 +430,7 @@ namespace Beagle.Daemon
 				DirectoryInfo dir = (DirectoryInfo) pending_directories.Dequeue ();
 
 				if (! arg_disable_directories)
-					AddToRequest (pending_request, DirectoryToIndexable (dir, modified_directories));
+					AddToRequest (DirectoryToIndexable (dir, modified_directories));
 
 				try {
 					if (arg_recursive)
@@ -367,7 +442,7 @@ namespace Beagle.Daemon
 					foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir))
 						if (!Ignore (file)
 						    && !FileSystem.IsSpecialFile (file.FullName)) {
-							AddToRequest (pending_request, FileToIndexable (file));
+							AddToRequest (FileToIndexable (file));
 							count_files ++;
 						}
 				
@@ -406,17 +481,18 @@ namespace Beagle.Daemon
 					// remove
 					Uri uri = UriFu.PathToFileUri (info.FullName);
 					indexable = new Indexable (IndexableType.Remove, uri);
-					AddToRequest (pending_request, indexable);
+					AddToRequest (indexable);
 				}
 			}
-			
+
+			bool reschedule = false;
 			// Call Flush until our request is empty.  We have to do this in a loop
-			// because children can get added back to the pending request in a flush.
-			while (pending_request.Count > 0) {
+			// because Flush happens in a batch size and some indexables might generate more indexables
+			while (reschedule || pending_request.Count > 0) {
 				if (Shutdown.ShutdownRequested)
 					break;
 
-				FlushIndexer (driver, pending_request);
+				reschedule = FlushIndexer (driver);
 			}
 
 			backing_fa_store.Flush ();
@@ -430,7 +506,7 @@ namespace Beagle.Daemon
 		
 		/////////////////////////////////////////////////////////////////
 
-		static void AddToRequest (IndexerRequest request, Indexable indexable)
+		static void AddToRequest (Indexable indexable)
 		{
 			if (indexable == null)
 				return;
@@ -443,46 +519,58 @@ namespace Beagle.Daemon
 			if (arg_tag != null)
 				indexable.AddProperty (Property.NewUnsearched("Tag", arg_tag));
 
-			if (arg_source == null) {
-				DirectoryInfo dir = new DirectoryInfo (StringFu.SanitizePath (arg_output));
-				arg_source = dir.Name;
-			}
-
 			indexable.Source = arg_source;
 
-			request.Add (indexable);
+			pending_request.Add (indexable);
+			bool reschedule = false;
 
-			if (! Shutdown.ShutdownRequested && request.Count >= BATCH_SIZE) {
-				Logger.Log.Debug ("Flushing driver, {0} items in queue", request.Count);
-				FlushIndexer (driver, request);
-				// FlushIndexer clears the pending_request
-			}
+			do {
+				if (Shutdown.ShutdownRequested)
+					break;
 
-			
-			// Super Lame Hack: gtk-sharp up to 2.10 requires a main loop
-			// to dispose of any managed wrappers around GObjects.  Since
-			// we don't have one, we'll process all the pending items in
-			// a loop here.  This is particularly an issue with maildirs,
-			// because we need the loop to clean up after GMime.  Without
-			// it, GMime's streams are never completely unref'd, the
-			// file descriptors aren't closed, and we run out and crash.
-			while (GLib.MainContext.Pending ())
-				GLib.MainContext.Iteration ();
+				if (! reschedule && pending_request.Count < BATCH_SIZE)
+					break;
+
+				if (reschedule)
+					Logger.Log.Debug ("Continuing indexing indexer generated indexables");
+				else
+					Logger.Log.Debug ("Flushing driver, {0} items in queue", pending_request.Count);
+
+				reschedule = FlushIndexer (driver);
+				Console.WriteLine (reschedule);
+
+				// Super Lame Hack: gtk-sharp up to 2.10 requires a main loop
+				// to dispose of any managed wrappers around GObjects.  Since
+				// we don't have one, we'll process all the pending items in
+				// a loop here.  This is particularly an issue with maildirs,
+				// because we need the loop to clean up after GMime.  Without
+				// it, GMime's streams are never completely unref'd, the
+				// file descriptors aren't closed, and we run out and crash.
+				while (GLib.MainContext.Pending ())
+					GLib.MainContext.Iteration ();
+
+			} while (reschedule);
 		}
 
-		static IndexerReceipt [] FlushIndexer (IIndexer indexer, IndexerRequest request)
+		// This is mostly a copy of LuceneQueryable.Flush + FSQ.PostAddHooks/PostRemoveHook
+		static bool FlushIndexer (IIndexer indexer)
 		{
+			IndexerRequest flushed_request;
+			if (pending_request.IsEmpty)
+				return false;
+
+			flushed_request = pending_request;
+			pending_request = new IndexerRequest ();
+
 			IndexerReceipt [] receipts;
-			receipts = indexer.Flush (request);
+			receipts = indexer.Flush (flushed_request);
 
 			// Flush will return null if it encounters a shutdown during flushing
 			if (receipts == null)
-				return null;
-
-			ArrayList pending_children;
-			pending_children = new ArrayList ();
+				return false;
 
 			fa_store.BeginTransaction ();
+			bool indexer_indexable_receipt = false;
 
 			foreach (IndexerReceipt raw_r in receipts) {
 
@@ -490,14 +578,19 @@ namespace Beagle.Daemon
 					// Update the file attributes 
 					IndexerAddedReceipt r = (IndexerAddedReceipt) raw_r;
 
-					Indexable indexable = request.GetByUri (r.Uri);
+					Indexable indexable = flushed_request.RetrieveRequestIndexable (r);
+
+					if (indexable == null) {
+						Logger.Log.Debug ("Should not happen! Previously requested indexable with id #{0} has eloped!", r.Id);
+						continue;
+					}
 
 					// We don't need to write out any file attributes for
 					// children.
 					if (indexable.ParentUri != null) 
 						continue;
 
-					string path = r.Uri.LocalPath;
+					string path = indexable.Uri.LocalPath;
 					
 					FileAttributes attr;
 					attr = fa_store.ReadOrCreate (path);
@@ -512,35 +605,36 @@ namespace Beagle.Daemon
 					// Update the file attributes 
 					IndexerRemovedReceipt r = (IndexerRemovedReceipt) raw_r;
 
-					Indexable indexable = request.GetByUri (r.Uri);
+					Indexable indexable = flushed_request.RetrieveRequestIndexable (r);
+					if (indexable == null) { // Should never happen
+						Log.Warn ("Unable to match indexable-remove #{0} to any request!", r.Id);
+						continue;
+					}
 
-					string path = r.Uri.LocalPath;
+					string path = indexable.Uri.LocalPath;
 					Logger.Log.Debug ("Removing: '{0}'", path);
 					fa_store.Drop (path);
 
-				} else if (raw_r is IndexerChildIndexablesReceipt) {
-					// Add any child indexables back into our indexer
-					IndexerChildIndexablesReceipt r = (IndexerChildIndexablesReceipt) raw_r;
-					pending_children.AddRange (r.Children);
+				} else if (raw_r is IndexerIndexablesReceipt) {
+					indexer_indexable_receipt = true;
 				}
 			}
 
+			pending_request.DeferredIndexables = flushed_request.DeferredIndexables;
+
+			// Reschedule if some indexable generated more indexables
+			if (indexer_indexable_receipt) {
+				pending_request.ContinueIndexing = true;
+				return true;
+			}
+
 			fa_store.CommitTransaction ();
-
-			request.Clear (); // clear out the old request
-
-			if (Shutdown.ShutdownRequested)
-				return receipts;
-
-			foreach (Indexable i in pending_children) // and then add the children
-				AddToRequest (request, i);
-			
-			return receipts;
+			return false;
 		}
 
 		static Indexable FileToIndexable (FileInfo file)
 		{
-			if (!file.Exists || Ignore (file) || fa_store.IsUpToDateAndFiltered (file.FullName))
+			if (!file.Exists || fa_store.IsUpToDateAndFiltered (file.FullName))
 				return null;
 
 			// Create the indexable and add the standard properties we
@@ -765,12 +859,9 @@ namespace Beagle.Daemon
 		
 		static void PrintUsage ()
 		{
-			string usage = 
-				"beagle-build-index: Build an index.\n" + 
-				"Web page: http://www.gnome.org/projects/beagle\n" +
-				"Copyright (C) 2005-2006 Novell, Inc.\n\n";
+			VersionFu.PrintHeader ();
 			
-			usage += 
+			string usage = 
 				"Usage: beagle-build-index [OPTIONS] --target <index_path> <path> [path]\n\n" +
 
 				"** WARNING **\n" +
@@ -785,12 +876,15 @@ namespace Beagle.Daemon
 				"  --tag [tag]\t\t\tTag index data for identification.\n" + 
 				"  --recursive\t\t\tCrawl source path recursivly.\n" + 
 				"  --enable-deletion\t\tRemove deleted files and directories from index.\n" +
+				"                   \t\tIndex should be created and always updated with this option.\n" +
+				"                   \t\tOption only works if --disable-directories is NOT specified.\n" +
 				"  --enable-text-cache\t\tBuild text-cache of documents used for snippets.\n" +
 				"  --disable-directories\t\tDon't add directories to the index.\n" +
 				"  --disable-filtering\t\tDisable all filtering of files. Only index attributes.\n" + 
 				"  --allow-pattern [pattern]\tOnly allow files that match the pattern to be indexed.\n" + 
 				"  --deny-pattern [pattern]\tKeep any files that match the pattern from being indexed.\n" + 
 				"  --disable-restart\t\tDon't restart when memory usage gets above a certain threshold.\n" +
+				"  --disable-on-battery\t\tDisable indexer while on battery power.\n" +
 				"  --debug\t\t\tEcho verbose debugging information.\n\n";
 
 			
@@ -816,21 +910,13 @@ namespace Beagle.Daemon
 			if (FileSystem.IsSpecialFile (file.FullName))
 				return true;
 
-			if (allowed_patterns.Count > 0) {
-				foreach (ExcludeItem pattern in allowed_patterns)
-					if (pattern.IsMatch (file.Name))
-						return false;
-				
-				return true;
-			}
+			if (allowed_regex != null)
+				return ! allowed_regex.IsMatch (file.Name);
 
-			foreach (ExcludeItem pattern in denied_patterns)
-				if (pattern.IsMatch (file.Name))
-					return true;
+			if (denied_regex == null)
+				return false;
 
-			// FIXME: Add more stuff here
-			
-			return false;
+			return denied_regex.IsMatch (file.Name);
 		}
 	}
 }
