@@ -1,7 +1,7 @@
 //
 // QueryDriver.cs
 //
-// Copyright (C) 2004 Novell, Inc.
+// Copyright (C) 2004-2007 Novell, Inc.
 //
 
 //
@@ -27,10 +27,12 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using System.Reflection;
 using System.Text;
 using System.Threading;
 using Beagle.Util;
+
 namespace Beagle.Daemon {
 	
 	public class QueryDriver {
@@ -54,15 +56,18 @@ namespace Beagle.Daemon {
 			// set flag here to stop Allow() from calling ReadBackendsFromConf() again
 			done_reading_conf = true;
 
+			Config config = Conf.Get (Conf.Names.DaemonConfig);
+
 			// To allow static indexes, "static" should be in allowed_queryables
-			if (Conf.Daemon.AllowStaticBackend)
+			if (config.GetOption (Conf.Names.AllowStaticBackend, false))
 				Allow ("static");
 
-			if (Conf.Daemon.DeniedBackends == null)
+			List<string[]> values = config.GetListOptionValues (Conf.Names.DeniedBackends);
+			if (values == null)
 				return;
 			
-			foreach (string name in Conf.Daemon.DeniedBackends)
-				denied_queryables.Add (name.ToLower ());
+			foreach (string[] name in values)
+				denied_queryables.Add (name [0].ToLower ());
 		}
 
 		static public void OnlyAllow (string name)
@@ -252,8 +257,11 @@ namespace Beagle.Daemon {
 
 			if (UseQueryable ("static")) {
 				Logger.Log.Info ("Loading user-configured static indexes.");
-				foreach (string path in Conf.Daemon.StaticQueryables)
-					static_queryables.Add (path);
+				List<string[]> values = Conf.Daemon.GetListOptionValues (Conf.Names.StaticQueryables);
+				if (values != null) {
+					foreach (string[] path in values)
+						static_queryables.Add (path [0]);
+				}
 			}
 
 			foreach (string path in static_queryables) {
@@ -287,21 +295,40 @@ namespace Beagle.Daemon {
 				Logger.Log.Error (e, "Caught exception while instantiating static queryable: {0}", index_dir.Name);
 				return false;
 			}
-			
-			if (static_queryable != null) {
-				QueryableFlavor flavor = new QueryableFlavor ();
-				flavor.Name = index_dir.Name;
-				flavor.Domain = query_domain;
-				
-				Queryable queryable = new Queryable (flavor, static_queryable);
-				queryables.Add (queryable);
-				
-				iqueryable_to_queryable [static_queryable] = queryable;
 
-				return true;
+			if (static_queryable == null)
+				return false;
+
+			// Load StaticIndex.xml from index_dir.FullName/config
+			string config_file_path = Path.Combine (index_dir.FullName, "StaticIndex.xml");
+			Config static_index_config;
+			try {
+				static_index_config = Conf.LoadFrom (config_file_path);
+				if (static_index_config == null) {
+					Log.Error ("Unable to read config from {0}", config_file_path);
+					return false;
+				}
+			} catch (Exception e) {
+				Log.Error (e, "Caught exception while reading config from {0}", config_file_path);
+				return false;
 			}
 
-			return false;
+			string source = static_index_config.GetOption ("Source", null);
+			if (source == null) {
+				Log.Error ("Invalid config file: {0}", config_file_path);
+				return false;
+			}
+
+			QueryableFlavor flavor = new QueryableFlavor ();
+			flavor.Name = source;
+			flavor.Domain = query_domain;
+
+			Queryable queryable = new Queryable (flavor, static_queryable);
+			queryables.Add (queryable);
+
+			iqueryable_to_queryable [static_queryable] = queryable;
+
+			return true;
 		}
 
 		////////////////////////////////////////////////////////
@@ -316,6 +343,8 @@ namespace Beagle.Daemon {
 			assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 		}
 
+		////////////////////////////////////////////////////////
+
 		private static bool queryables_started = false;
 
 		static public void Start ()
@@ -351,9 +380,46 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Debug ("Starting queryables");
 
-			foreach (Queryable q in queryables) {
-				Logger.Log.Info ("Starting backend: '{0}'", q.Name);
-				q.Start ();
+			ArrayList started_queryables = new ArrayList ();
+			ArrayList delayed_queryables = new ArrayList ();
+
+			ICollection queryables_to_start = queryables;
+			int last_count;
+
+			do {
+				foreach (Queryable q in queryables_to_start) {
+					bool ready_to_start = true;
+
+					if (q.DependsOn != null && started_queryables.IndexOf (q.DependsOn) == -1) {
+						Log.Info ("Delaying backend '{0}' until after backend '{1}'",
+							  q.Name, q.DependsOn);
+						ready_to_start = false;
+						delayed_queryables.Add (q);
+					}
+
+					if (! ready_to_start)
+						continue;
+
+					Logger.Log.Info ("Starting backend: '{0}'", q.Name);
+					q.Start ();
+
+					started_queryables.Add (q.Name);
+				}
+
+				last_count = queryables_to_start.Count;
+
+				queryables_to_start = delayed_queryables;
+				delayed_queryables = new ArrayList ();
+			} while (queryables_to_start.Count > 0 && queryables_to_start.Count != last_count);
+
+			if (queryables_to_start.Count > 0) {
+				Log.Info ("Unable to start backends due to missing or circular dependencies:");
+
+				foreach (Queryable q in queryables_to_start) {
+					Log.Info ("  - {0}", q.Name);
+					queryables.Remove (q);
+					iqueryable_to_queryable.Remove (q.IQueryable);
+				}
 			}
 
 			queryables_started = true;
@@ -361,7 +427,9 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		static public string ListBackends ()
+		////////////////////////////////////////////////////////
+
+		public static string ListBackends ()
 		{
 			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 
@@ -452,9 +520,13 @@ namespace Beagle.Daemon {
 					       QueryResult          result,
 					       IQueryableChangeData change_data)
 		{
-			if (queryable.AcceptQuery (query)) {
-				QueryClosure qc = new QueryClosure (queryable, query, result, change_data);
-				result.AttachWorker (qc);
+			try {
+				if (queryable.AcceptQuery (query)) {
+					QueryClosure qc = new QueryClosure (queryable, query, result, change_data);
+					result.AttachWorker (qc);
+				}
+			} catch (Exception ex) {
+				Logger.Log.Warn (ex, "Caught exception calling DoOneQuery on '{0}'", queryable.Name);
 			}
 		}
 
@@ -554,8 +626,7 @@ namespace Beagle.Daemon {
 			return search_term_response;
 		}
 
-		static private void QueryEachQueryable (Query       query,
-							QueryResult result)
+		static private void QueryEachQueryable (Query query, QueryResult result)
 		{
 			// The extra pair of calls to WorkerStart/WorkerFinished ensures:
 			// (1) that the QueryResult will fire the StartedEvent
